@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The B+ tree database API of Tokyo Cabinet
- *                                                      Copyright (C) 2006-2007 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -28,9 +28,9 @@ __TCBDB_CLINKAGEBEGIN
 
 
 #include <stdlib.h>
-#include <time.h>
 #include <stdbool.h>
 #include <stdint.h>
+#include <time.h>
 #include <tcutil.h>
 #include <tchdb.h>
 
@@ -83,6 +83,7 @@ typedef struct {                         /* type of structure for a B+ tree data
   char *rbopaque;                        /* opaque for rollback */
   int64_t cnt_saveleaf;                  /* tesing counter for leaf save times */
   int64_t cnt_loadleaf;                  /* tesing counter for leaf load times */
+  int64_t cnt_killleaf;                  /* tesing counter for leaf kill times */
   int64_t cnt_adjleafc;                  /* tesing counter for node cache adjust times */
   int64_t cnt_savenode;                  /* tesing counter for node save times */
   int64_t cnt_loadnode;                  /* tesing counter for node load times */
@@ -480,14 +481,27 @@ TCLIST *tcbdbrange2(TCBDB *bdb, const char *bkstr, bool binc,
 
 /* Get forward matching keys in a B+ tree database object.
    `bdb' specifies the B+ tree database object.
-   `prefix' specifies the prefix of the corresponding keys.
+   `pbuf' specifies the pointer to the region of the prefix.
+   `psiz' specifies the size of the region of the prefix.
    `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
    specified.
-   The return value is a list object of the keys of the corresponding records.  This function
-   does never fail and return an empty list even if no record corresponds.
-   Because the object of the return value is created with the function `tclistnew', it should
-   be deleted with the function `tclistdel' when it is no longer in use. */
-TCLIST *tcbdbrange3(TCBDB *bdb, const char *prefix, int max);
+   The return value is a list object of the corresponding keys.  This function does never fail
+   and return an empty list even if no key corresponds.
+   Because the object of the return value is created with the function `tclistnew', it should be
+   deleted with the function `tclistdel' when it is no longer in use. */
+TCLIST *tcbdbfwmkeys(TCBDB *bdb, const void *pbuf, int psiz, int max);
+
+
+/* Get forward matching string keys in a B+ tree database object.
+   `bdb' specifies the B+ tree database object.
+   `pstr' specifies the string of the prefix.
+   `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
+   specified.
+   The return value is a list object of the corresponding keys.  This function does never fail
+   and return an empty list even if no key corresponds.
+   Because the object of the return value is created with the function `tclistnew', it should be
+   deleted with the function `tclistdel' when it is no longer in use. */
+TCLIST *tcbdbfwmkeys2(TCBDB *bdb, const char *pstr, int max);
 
 
 /* Synchronize updated contents of a B+ tree database object with the file and the device.
@@ -767,6 +781,15 @@ bool tcbdbcurrec(BDBCUR *cur, TCXSTR *kxstr, TCXSTR *vxstr);
  *************************************************************************************************/
 
 
+/* Set the error code of a B+ tree database object.
+   `bdb' specifies the B+ tree database object.
+   `ecode' specifies the error code.
+   `file' specifies the file name of the code.
+   `line' specifies the line number of the code.
+   `func' specifies the function name of the code. */
+void tcbdbsetecode(TCBDB *bdb, int ecode, const char *filename, int line, const char *func);
+
+
 /* Set the file descriptor for debugging output.
    `bdb' specifies the B+ tree database object.
    `fd' specifies the file descriptor for debugging output. */
@@ -968,6 +991,10 @@ int tcbdbcmpint32(const char *aptr, int asiz, const char *bptr, int bsiz, void *
 int tcbdbcmpint64(const char *aptr, int asiz, const char *bptr, int bsiz, void *op);
 
 
+/* tricks for backward compatibility */
+#define tcbdbrange3 tcbdbfwmkeys2
+
+
 
 __TCBDB_CLINKAGEEND
 #endif                                   /* duplication check */
