@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The command line utility of the B+ tree database API
- *                                                      Copyright (C) 2006-2007 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -15,12 +15,9 @@
 
 
 #include <tcutil.h>
-#include <tchdb.h>
 #include <tcbdb.h>
 #include "myconf.h"
 
-#define RECBUFSIZ      32                // buffer for records
-
 
 /* global variables */
 const char *g_progname;                  // program name
@@ -52,7 +49,7 @@ static int procout(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int
 static int procget(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int omode,
                    bool px, bool pz);
 static int proclist(const char *path, BDBCMP cmp, int omode, int max, bool pv, bool px, bool bk,
-                    const char *jstr, const char *bstr, const char *estr, const char *pstr);
+                    const char *jstr, const char *bstr, const char *estr, const char *fmstr);
 static int procoptimize(const char *path, int lmemb, int nmemb,
                         int bnum, int apow, int fpow, BDBCMP cmp, int opts, int omode);
 static int procimporttsv(const char *path, const char *file, int omode, bool sc);
@@ -105,7 +102,7 @@ static void usage(void){
   fprintf(stderr, "  %s out [-cd|-ci|-cj] [-nl|-nb] [-sx] path key\n", g_progname);
   fprintf(stderr, "  %s get [-cd|-ci|-cj] [-nl|-nb] [-sx] [-px] [-pz] path key\n", g_progname);
   fprintf(stderr, "  %s list [-cd|-ci|-cj] [-nl|-nb] [-m num] [-bk] [-pv] [-px] [-j str]"
-          " [-rb bkey ekey] [-rp str] path\n", g_progname);
+          " [-rb bkey ekey] [-fm str] path\n", g_progname);
   fprintf(stderr, "  %s optimize [-cd|-ci|-cj] [-tl] [-td|-tb] [-tz] [-nl|-nb] path"
           " [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
   fprintf(stderr, "  %s importtsv [-nl|-nb] [-sc] path [file]\n", g_progname);
@@ -435,7 +432,7 @@ static int runlist(int argc, char **argv){
   char *jstr = NULL;
   char *bstr = NULL;
   char *estr = NULL;
-  char *pstr = NULL;
+  char *fmstr = NULL;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-cd")){
@@ -465,9 +462,9 @@ static int runlist(int argc, char **argv){
         bstr = argv[i];
         if(++i >= argc) usage();
         estr = argv[i];
-      } else if(!strcmp(argv[i], "-rp")){
+      } else if(!strcmp(argv[i], "-fm")){
         if(++i >= argc) usage();
-        pstr = argv[i];
+        fmstr = argv[i];
       } else {
         usage();
       }
@@ -478,7 +475,7 @@ static int runlist(int argc, char **argv){
     }
   }
   if(!path) usage();
-  int rv = proclist(path, cmp, omode, max, pv, px, bk, jstr, bstr, estr, pstr);
+  int rv = proclist(path, cmp, omode, max, pv, px, bk, jstr, bstr, estr, fmstr);
   return rv;
 }
 
@@ -639,6 +636,10 @@ static int procinform(const char *path, int omode){
     printf("used bucket number: %lld\n", (long long)tcbdbbnumused(bdb));
   printf("alignment: %u\n", tcbdbalign(bdb));
   printf("free block pool: %u\n", tcbdbfbpmax(bdb));
+  printf("inode number: %lld\n", (long long)tcbdbinode(bdb));
+  char date[48];
+  tcdatestrwww(tcbdbmtime(bdb), INT_MAX, date);
+  printf("modified time: %s\n", date);
   uint8_t opts = tcbdbopts(bdb);
   printf("options:");
   if(opts & BDBTLARGE) printf(" large");
@@ -766,7 +767,7 @@ static int procget(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int
 
 /* perform list command */
 static int proclist(const char *path, BDBCMP cmp, int omode, int max, bool pv, bool px, bool bk,
-                    const char *jstr, const char *bstr, const char *estr, const char *pstr){
+                    const char *jstr, const char *bstr, const char *estr, const char *fmstr){
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   if(cmp && !tcbdbsetcmpfunc(bdb, cmp, NULL)) printerr(bdb);
@@ -776,8 +777,8 @@ static int proclist(const char *path, BDBCMP cmp, int omode, int max, bool pv, b
     return 1;
   }
   bool err = false;
-  if(bstr || pstr){
-    TCLIST *keys = pstr ? tcbdbrange3(bdb, pstr, max) :
+  if(bstr || fmstr){
+    TCLIST *keys = fmstr ? tcbdbfwmkeys2(bdb, fmstr, max) :
       tcbdbrange(bdb, bstr, strlen(bstr), true, estr, strlen(estr), true, max);
     int cnt = 0;
     for(int i = 0; i < tclistnum(keys); i++){
@@ -945,7 +946,7 @@ static int procimporttsv(const char *path, const char *file, int omode, bool sc)
 /* perform version command */
 static int procversion(void){
   printf("Tokyo Cabinet version %s (%d:%s)\n", tcversion, _TC_LIBVER, _TC_FORMATVER);
-  printf("Copyright (C) 2006-2007 Mikio Hirabayashi\n");
+  printf("Copyright (C) 2006-2008 Mikio Hirabayashi\n");
   return 0;
 }
 
