@@ -1,6 +1,6 @@
 /*************************************************************************************************
- * The test cases of the hash database API
- *                                                      Copyright (C) 2006-2007 Mikio Hirabayashi
+ * The test cases of the B+ tree database API
+ *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -15,7 +15,6 @@
 
 
 #include <tcutil.h>
-#include <tchdb.h>
 #include <tcbdb.h>
 #include "myconf.h"
 
@@ -55,6 +54,7 @@ typedef struct {                         // type of structure for typical thread
   TCBDB *bdb;
   int rnum;
   bool nc;
+  int rratio;
   int id;
 } TARGTYPICAL;
 
@@ -71,6 +71,7 @@ static void iprintf(const char *format, ...);
 static void eprint(TCBDB *bdb, const char *func);
 static void mprint(TCBDB *bdb);
 static int myrand(int range);
+static int myrandnd(int range);
 static int runwrite(int argc, char **argv);
 static int runread(int argc, char **argv);
 static int runremove(int argc, char **argv);
@@ -82,7 +83,7 @@ static int procread(const char *path, int tnum, int omode, bool wb, bool rnd);
 static int procremove(const char *path, int tnum, int omode, bool rnd);
 static int procwicked(const char *path, int tnum, int rnum, int opts, int omode, bool nc);
 static int proctypical(const char *path, int tnum, int rnum, int lmemb, int nmemb,
-                       int bnum, int apow, int fpow, int opts, int omode, bool nc);
+                       int bnum, int apow, int fpow, int opts, int omode, bool nc, int rratio);
 static void *threadwrite(void *targ);
 static void *threadread(void *targ);
 static void *threadremove(void *targ);
@@ -96,7 +97,7 @@ int main(int argc, char **argv){
   g_dbgfd = -1;
   const char *ebuf = getenv("TCDBGFD");
   if(ebuf) g_dbgfd = atoi(ebuf);
-  srand((unsigned int)(tctime() * 100) % UINT_MAX);
+  srand((unsigned int)(tctime() * 1000) % UINT_MAX);
   if(argc < 2) usage();
   int rv = 0;
   if(!strcmp(argv[1], "write")){
@@ -127,7 +128,7 @@ static void usage(void){
   fprintf(stderr, "  %s remove [-nl|-nb] [-rnd] path tnum\n", g_progname);
   fprintf(stderr, "  %s wicked [-tl] [-td|-tb] [-nl|-nb] [-nc] path tnum rnum\n",
           g_progname);
-  fprintf(stderr, "  %s typical [-tl] [-td|-tb] [-nl|-nb] [-nc] path tnum rnum"
+  fprintf(stderr, "  %s typical [-tl] [-td|-tb] [-nl|-nb] [-nc] [-rr] path tnum rnum"
           " [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
   fprintf(stderr, "\n");
   exit(1);
@@ -164,6 +165,7 @@ static void mprint(TCBDB *bdb){
   iprintf("used bucket number: %lld\n", (long long)tcbdbbnumused(bdb));
   iprintf("cnt_saveleaf: %lld\n", (long long)bdb->cnt_saveleaf);
   iprintf("cnt_loadleaf: %lld\n", (long long)bdb->cnt_loadleaf);
+  iprintf("cnt_killleaf: %lld\n", (long long)bdb->cnt_killleaf);
   iprintf("cnt_adjleafc: %lld\n", (long long)bdb->cnt_adjleafc);
   iprintf("cnt_savenode: %lld\n", (long long)bdb->cnt_savenode);
   iprintf("cnt_loadnode: %lld\n", (long long)bdb->cnt_loadnode);
@@ -181,6 +183,7 @@ static void mprint(TCBDB *bdb){
   iprintf("cnt_appenddrp: %lld\n", (long long)bdb->hdb->cnt_appenddrp);
   iprintf("cnt_deferdrp: %lld\n", (long long)bdb->hdb->cnt_deferdrp);
   iprintf("cnt_flushdrp: %lld\n", (long long)bdb->hdb->cnt_flushdrp);
+  iprintf("cnt_adjrecc: %lld\n", (long long)bdb->hdb->cnt_adjrecc);
 }
 
 
@@ -190,6 +193,13 @@ static int myrand(int range){
 }
 
 
+/* get a random number based on normal distribution */
+static int myrandnd(int range){
+  int num = (int)tcdrandnd(range >> 1, range / 10);
+  return (num < 0 || num >= range) ? 0 : num;
+}
+
+
 /* parse arguments of write command */
 static int runwrite(int argc, char **argv){
   char *path = NULL;
@@ -379,6 +389,7 @@ static int runtypical(int argc, char **argv){
   char *fstr = NULL;
   int opts = 0;
   int omode = 0;
+  int rratio = -1;
   bool nc = false;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
@@ -394,6 +405,9 @@ static int runtypical(int argc, char **argv){
         omode |= BDBOLCKNB;
       } else if(!strcmp(argv[i], "-nc")){
         nc = true;
+      } else if(!strcmp(argv[i], "-rr")){
+        if(++i >= argc) usage();
+        rratio = atoi(argv[i]);
       } else {
         usage();
       }
@@ -426,7 +440,7 @@ static int runtypical(int argc, char **argv){
   int bnum = bstr ? atoi(bstr) : -1;
   int apow = astr ? atoi(astr) : -1;
   int fpow = fstr ? atoi(fstr) : -1;
-  int rv = proctypical(path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, omode, nc);
+  int rv = proctypical(path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, omode, nc, rratio);
   return rv;
 }
 
@@ -492,7 +506,7 @@ static int procwrite(const char *path, int tnum, int rnum, int lmemb, int nmemb,
     err = true;
   }
   tcbdbdel(bdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -556,7 +570,7 @@ static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
     err = true;
   }
   tcbdbdel(bdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -617,7 +631,7 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
     err = true;
   }
   tcbdbdel(bdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -725,7 +739,7 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
     err = true;
   }
   tcbdbdel(bdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -733,10 +747,10 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
 
 /* perform typical command */
 static int proctypical(const char *path, int tnum, int rnum, int lmemb, int nmemb,
-                       int bnum, int apow, int fpow, int opts, int omode, bool nc){
+                       int bnum, int apow, int fpow, int opts, int omode, bool nc, int rratio){
   iprintf("<Typical Access Test>\n  path=%s  tnum=%d  rnum=%d  lmemb=%d  nmemb=%d"
-          "  bnum=%d  apow=%d  fpow=%d  opts=%d  omode=%d  nc=%d\n\n",
-          path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, omode, nc);
+          "  bnum=%d  apow=%d  fpow=%d  opts=%d  omode=%d  nc=%d  rratio=%d\n\n",
+          path, tnum, rnum, lmemb, nmemb, bnum, apow, fpow, opts, omode, nc, rratio);
   bool err = false;
   double stime = tctime();
   TCBDB *bdb = tcbdbnew();
@@ -759,6 +773,7 @@ static int proctypical(const char *path, int tnum, int rnum, int lmemb, int nmem
     targs[0].bdb = bdb;
     targs[0].rnum = rnum;
     targs[0].nc = nc;
+    targs[0].rratio = rratio;
     targs[0].id = 0;
     if(threadtypical(targs) != NULL) err = true;
   } else {
@@ -766,6 +781,7 @@ static int proctypical(const char *path, int tnum, int rnum, int lmemb, int nmem
       targs[i].bdb = bdb;
       targs[i].rnum = rnum;
       targs[i].nc = nc;
+      targs[i].rratio = rratio;
       targs[i].id = i;
       if(pthread_create(threads + i, NULL, threadtypical, targs + i) != 0){
         eprint(bdb, "pthread_create");
@@ -792,7 +808,7 @@ static int proctypical(const char *path, int tnum, int rnum, int lmemb, int nmem
     err = true;
   }
   tcbdbdel(bdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -835,7 +851,7 @@ static void *threadread(void *targ){
   int base = id * rnum;
   for(int i = 1; i <= rnum && !err; i++){
     char kbuf[RECBUFSIZ];
-    int ksiz = sprintf(kbuf, "%08d", base + (rnd ? myrand(i / 10 + 1) : i));
+    int ksiz = sprintf(kbuf, "%08d", base + (rnd ? myrandnd(i) : i));
     int vsiz;
     if(wb){
       int vsiz;
@@ -1107,7 +1123,7 @@ static void *threadwicked(void *targ){
       break;
     default:
       if(id == 0) putchar('@');
-      if(myrand(10000) == 0) srand((unsigned int)(tctime() * 100) % UINT_MAX);
+      if(myrand(10000) == 0) srand((unsigned int)(tctime() * 1000) % UINT_MAX);
       break;
     }
     if(!nc) tcglobalmutexunlock();
@@ -1135,15 +1151,17 @@ static void *threadtypical(void *targ){
   TCBDB *bdb = ((TARGTYPICAL *)targ)->bdb;
   int rnum = ((TARGTYPICAL *)targ)->rnum;
   bool nc = ((TARGTYPICAL *)targ)->nc;
+  int rratio = ((TARGTYPICAL *)targ)->rratio;
   int id = ((TARGTYPICAL *)targ)->id;
   bool err = false;
   TCMAP *map = (!nc && id == 0) ? tcmapnew2(rnum + 1) : NULL;
   int base = id * rnum;
+  int mrange = tclmax(50 + rratio, 100);
   BDBCUR *cur = tcbdbcurnew(bdb);
   for(int i = 1; !err && i <= rnum; i++){
     char buf[RECBUFSIZ];
-    int len = sprintf(buf, "%08d", base + myrand(i));
-    int rnd = myrand(100);
+    int len = sprintf(buf, "%08d", base + myrandnd(i));
+    int rnd = myrand(mrange);
     if(rnd < 10){
       if(!tcbdbput(bdb, buf, len, buf, len)){
         eprint(bdb, "tcbdbput");
