@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The hash database API of Tokyo Cabinet
- *                                                      Copyright (C) 2006-2007 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -14,8 +14,8 @@
  *************************************************************************************************/
 
 
-#include "tchdb.h"
 #include "tcutil.h"
+#include "tchdb.h"
 #include "myconf.h"
 
 #define HDBFILEMODE    00644             // permission of created files
@@ -46,7 +46,7 @@
 #define HDBFBPMGFREQ   256               // frequency to merge the free block pool
 #define HDBDRPUNIT     65536             // unit size of the delayed record pool
 #define HDBDRPLAT      2048              // latitude size of the delayed record pool
-#define HDBCACHEOUT    32                // number of records in a process of cacheout
+#define HDBCACHEOUT    128               // number of records in a process of cacheout
 
 typedef struct {                         // type of structure for a record
   uint64_t off;                          // offset of the record
@@ -86,14 +86,10 @@ enum {                                   // enumeration for duplication behavior
   ((TC_hdb)->mmtx ? tchdblockmethod((TC_hdb), (TC_wr)) : true)
 #define HDBUNLOCKMETHOD(TC_hdb) \
   ((TC_hdb)->mmtx ? tchdbunlockmethod(TC_hdb) : true)
-#define HDBLOCKCACHE(TC_hdb, TC_wr) \
-  ((TC_hdb)->mmtx ? tchdblockcache((TC_hdb), (TC_wr)) : true)
-#define HDBUNLOCKCACHE(TC_hdb) \
-  ((TC_hdb)->mmtx ? tchdbunlockcache(TC_hdb) : true)
-#define HDBLOCKFIO(TC_hdb) \
-  ((TC_hdb)->mmtx ? tchdblockfio(TC_hdb) : true)
-#define HDBUNLOCKFIO(TC_hdb) \
-  ((TC_hdb)->mmtx ? tchdbunlockfio(TC_hdb) : true)
+#define HDBLOCKDRP(TC_hdb) \
+  ((TC_hdb)->mmtx ? tchdblockdrp(TC_hdb) : true)
+#define HDBUNLOCKDRP(TC_hdb) \
+  ((TC_hdb)->mmtx ? tchdbunlockdrp(TC_hdb) : true)
 
 
 /* private function prototypes */
@@ -101,7 +97,7 @@ static bool tcseekwrite(TCHDB *hdb, off_t off, const void *buf, size_t size);
 static bool tcseekread(TCHDB *hdb, off_t off, void *buf, size_t size);
 static void tcdumpmeta(TCHDB *hdb, char *hbuf);
 static void tcloadmeta(TCHDB *hdb, const char *hbuf);
-static uint64_t tcgetprime(int num);
+static uint64_t tcgetprime(uint64_t num);
 static void tchdbclear(TCHDB *hdb);
 static int32_t tchdbpadsize(TCHDB *hdb);
 static void tchdbsetflag(TCHDB *hdb, int flag, bool sign);
@@ -141,10 +137,8 @@ static bool tchdboptimizeimpl(TCHDB *hdb, int64_t bnum, int8_t apow, int8_t fpow
 static bool tchdbvanishimpl(TCHDB *hdb);
 static bool tchdblockmethod(TCHDB *hdb, bool wr);
 static bool tchdbunlockmethod(TCHDB *hdb);
-static bool tchdblockcache(TCHDB *hdb, bool wr);
-static bool tchdbunlockcache(TCHDB *hdb);
-static bool tchdblockfio(TCHDB *hdb);
-static bool tchdbunlockfio(TCHDB *hdb);
+static bool tchdblockdrp(TCHDB *hdb);
+static bool tchdbunlockdrp(TCHDB *hdb);
 
 
 /* debugging function prototypes */
@@ -205,12 +199,10 @@ void tchdbdel(TCHDB *hdb){
   if(hdb->fd >= 0) tchdbclose(hdb);
   if(hdb->mmtx){
     pthread_key_delete(*(pthread_key_t *)hdb->eckey);
-    pthread_mutex_destroy(hdb->fmtx);
-    pthread_rwlock_destroy(hdb->cmtx);
+    pthread_mutex_destroy(hdb->dmtx);
     pthread_rwlock_destroy(hdb->mmtx);
     free(hdb->eckey);
-    free(hdb->fmtx);
-    free(hdb->cmtx);
+    free(hdb->dmtx);
     free(hdb->mmtx);
   }
   free(hdb);
@@ -238,20 +230,18 @@ bool tchdbsetmutex(TCHDB *hdb){
   pthread_mutexattr_t rma;
   pthread_mutexattr_init(&rma);
   TCMALLOC(hdb->mmtx, sizeof(pthread_rwlock_t));
-  TCMALLOC(hdb->cmtx, sizeof(pthread_rwlock_t));
-  TCMALLOC(hdb->fmtx, sizeof(pthread_mutex_t));
+  TCMALLOC(hdb->dmtx, sizeof(pthread_mutex_t));
   TCMALLOC(hdb->eckey, sizeof(pthread_key_t));
   if(pthread_mutexattr_settype(&rma, PTHREAD_MUTEX_RECURSIVE) != 0 ||
-     pthread_rwlock_init(hdb->mmtx, NULL) != 0 || pthread_rwlock_init(hdb->cmtx, NULL) != 0 ||
-     pthread_mutex_init(hdb->fmtx, &rma) != 0 || pthread_key_create(hdb->eckey, NULL) != 0){
+     pthread_rwlock_init(hdb->mmtx, NULL) != 0 || pthread_mutex_init(hdb->dmtx, &rma) != 0 ||
+     pthread_key_create(hdb->eckey, NULL) != 0){
+    tchdbsetecode(hdb, TCETHREAD, __FILE__, __LINE__, __func__);
     pthread_mutexattr_destroy(&rma);
     free(hdb->eckey);
-    free(hdb->fmtx);
-    free(hdb->cmtx);
+    free(hdb->dmtx);
     free(hdb->mmtx);
     hdb->eckey = NULL;
-    hdb->fmtx = NULL;
-    hdb->cmtx = NULL;
+    hdb->dmtx = NULL;
     hdb->mmtx = NULL;
     tcglobalmutexunlock();
     return false;
@@ -329,7 +319,7 @@ bool tchdbput(TCHDB *hdb, const void *kbuf, int ksiz, const void *vbuf, int vsiz
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
@@ -372,7 +362,7 @@ bool tchdbputkeep(TCHDB *hdb, const void *kbuf, int ksiz, const void *vbuf, int
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
@@ -414,7 +404,7 @@ bool tchdbputcat(TCHDB *hdb, const void *kbuf, int ksiz, const void *vbuf, int v
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
@@ -465,6 +455,7 @@ bool tchdbputcat2(TCHDB *hdb, const char *kstr, const char *vstr){
 bool tchdbputasync(TCHDB *hdb, const void *kbuf, int ksiz, const void *vbuf, int vsiz){
   assert(hdb && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
   if(!HDBLOCKMETHOD(hdb, true)) return false;
+  hdb->async = true;
   if(hdb->fd < 0 || !(hdb->omode & HDBOWRITER)){
     tchdbsetecode(hdb, TCEINVALID, __FILE__, __LINE__, __func__);
     HDBUNLOCKMETHOD(hdb);
@@ -509,7 +500,7 @@ bool tchdbout(TCHDB *hdb, const void *kbuf, int ksiz){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
@@ -535,7 +526,7 @@ void *tchdbget(TCHDB *hdb, const void *kbuf, int ksiz, int *sp){
     HDBUNLOCKMETHOD(hdb);
     return NULL;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return NULL;
   }
@@ -562,7 +553,7 @@ int tchdbget3(TCHDB *hdb, const void *kbuf, int ksiz, void *vbuf, int max){
     HDBUNLOCKMETHOD(hdb);
     return -1;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return -1;
   }
@@ -581,7 +572,7 @@ int tchdbvsiz(TCHDB *hdb, const void *kbuf, int ksiz){
     HDBUNLOCKMETHOD(hdb);
     return -1;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return -1;
   }
@@ -607,7 +598,7 @@ bool tchdbiterinit(TCHDB *hdb){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
@@ -626,7 +617,7 @@ void *tchdbiternext(TCHDB *hdb, int *sp){
     HDBUNLOCKMETHOD(hdb);
     return NULL;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return NULL;
   }
@@ -653,7 +644,7 @@ bool tchdbiternext3(TCHDB *hdb, TCXSTR *kxstr, TCXSTR *vxstr){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
@@ -663,6 +654,45 @@ bool tchdbiternext3(TCHDB *hdb, TCXSTR *kxstr, TCXSTR *vxstr){
 }
 
 
+/* Get forward matching keys in a hash database object. */
+TCLIST *tchdbfwmkeys(TCHDB *hdb, const void *pbuf, int psiz, int max){
+  assert(hdb && pbuf && psiz >= 0);
+  TCLIST* keys = tclistnew();
+  if(!HDBLOCKMETHOD(hdb, true)) return keys;
+  if(hdb->fd < 0){
+    tchdbsetecode(hdb, TCEINVALID, __FILE__, __LINE__, __func__);
+    HDBUNLOCKMETHOD(hdb);
+    return keys;
+  }
+  if(hdb->async && !tchdbflushdrp(hdb)){
+    HDBUNLOCKMETHOD(hdb);
+    return keys;
+  }
+  if(max < 0) max = INT_MAX;
+  uint64_t iter = hdb->iter;
+  tchdbiterinitimpl(hdb);
+  char *kbuf;
+  int ksiz;
+  while(TCLISTNUM(keys) < max && (kbuf = tchdbiternextimpl(hdb, &ksiz)) != NULL){
+    if(ksiz >= psiz && !memcmp(kbuf, pbuf, psiz)){
+      tclistpushmalloc(keys, kbuf, ksiz);
+    } else {
+      free(kbuf);
+    }
+  }
+  hdb->iter = iter;
+  HDBUNLOCKMETHOD(hdb);
+  return keys;
+}
+
+
+/* Get forward matching string keys in a hash database object. */
+TCLIST *tchdbfwmkeys2(TCHDB *hdb, const char *pstr, int max){
+  assert(hdb && pstr);
+  return tchdbfwmkeys(hdb, pstr, strlen(pstr), max);
+}
+
+
 /* Synchronize updated contents of a hash database object with the file and the device. */
 bool tchdbsync(TCHDB *hdb){
   assert(hdb);
@@ -672,7 +702,7 @@ bool tchdbsync(TCHDB *hdb){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
@@ -691,7 +721,7 @@ bool tchdboptimize(TCHDB *hdb, int64_t bnum, int8_t apow, int8_t fpow, uint8_t o
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
@@ -710,7 +740,7 @@ bool tchdbvanish(TCHDB *hdb){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
@@ -723,13 +753,17 @@ bool tchdbvanish(TCHDB *hdb){
 /* Copy the database file of a hash database object. */
 bool tchdbcopy(TCHDB *hdb, const char *path){
   assert(hdb && path);
-  if(!HDBLOCKMETHOD(hdb, true)) return false;
+  if(!HDBLOCKMETHOD(hdb, false)) return false;
   if(hdb->fd < 0 || !(hdb->omode & HDBOWRITER)){
     tchdbsetecode(hdb, TCEINVALID, __FILE__, __LINE__, __func__);
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
-  if(!tchdbflushdrp(hdb)){
+  if(hdb->async && !tchdbflushdrp(hdb)){
+    HDBUNLOCKMETHOD(hdb);
+    return false;
+  }
+  if(!HDBLOCKDRP(hdb)){
     HDBUNLOCKMETHOD(hdb);
     return false;
   }
@@ -740,7 +774,24 @@ bool tchdbcopy(TCHDB *hdb, const char *path){
     if(!tchdbmemsync(hdb, false)) err = true;
   }
   if(*path == '@'){
-    char *cmd = tcsprintf("%s %s", path + 1, hdb->path);
+    int len = strlen(hdb->path);
+    char name[len*2+1];
+    char *wp = name;
+    for(int i = 0; i < len; i++){
+      switch(hdb->path[i]){
+      case '\\':
+      case '$':
+        *(wp++) = '\\';
+        *(wp++) = hdb->path[i];
+        break;
+      default:
+        *(wp++) = hdb->path[i];
+        break;
+      }
+    }
+    *wp = '\0';
+    char *cmd = tcsprintf("%s \"%s\" \"%llu\"",
+                          path + 1, name, (unsigned long long)(tctime() * 1000000));
     if(system(cmd) != 0) err = true;
     free(cmd);
   } else {
@@ -750,8 +801,9 @@ bool tchdbcopy(TCHDB *hdb, const char *path){
     }
   }
   hdb->flags |= HDBFOPEN;
+  HDBUNLOCKDRP(hdb);
   HDBUNLOCKMETHOD(hdb);
-  return err ? false : true;
+  return !err;
 }
 
 
@@ -872,7 +924,7 @@ bool tchdbmemsync(TCHDB *hdb, bool phys){
       err = true;
     }
   }
-  return err ? false : true;
+  return !err;
 }
 
 
@@ -1023,18 +1075,26 @@ uint64_t tchdbbnumused(TCHDB *hdb){
    The return value is true if successful, else, it is false. */
 static bool tcseekwrite(TCHDB *hdb, off_t off, const void *buf, size_t size){
   assert(hdb && off >= 0 && buf && size >= 0);
-  if(!HDBLOCKFIO(hdb)) return false;
-  if(lseek(hdb->fd, off, SEEK_SET) == -1){
-    tchdbsetecode(hdb, TCESEEK, __FILE__, __LINE__, __func__);
-    HDBUNLOCKFIO(hdb);
-    return false;
+  while(true){
+    int wb = pwrite(hdb->fd, buf, size, off);
+    if(wb >= size){
+      return true;
+    } else if(wb > 0){
+      buf = (char *)buf + wb;
+      size -= wb;
+      off += wb;
+    } else if(wb == -1){
+      if(errno != EINTR){
+        tchdbsetecode(hdb, TCEWRITE, __FILE__, __LINE__, __func__);
+        return false;
+      }
+    } else {
+      if(size > 0){
+        tchdbsetecode(hdb, TCEWRITE, __FILE__, __LINE__, __func__);
+        return false;
+      }
+    }
   }
-  if(!tcwrite(hdb->fd, buf, size)){
-    tchdbsetecode(hdb, TCEWRITE, __FILE__, __LINE__, __func__);
-    HDBUNLOCKFIO(hdb);
-    return false;
-  }
-  HDBUNLOCKFIO(hdb);
   return true;
 }
 
@@ -1047,18 +1107,26 @@ static bool tcseekwrite(TCHDB *hdb, off_t off, const void *buf, size_t size){
    The return value is true if successful, else, it is false. */
 static bool tcseekread(TCHDB *hdb, off_t off, void *buf, size_t size){
   assert(hdb && off >= 0 && buf && size >= 0);
-  if(!HDBLOCKFIO(hdb)) return false;
-  if(lseek(hdb->fd, off, SEEK_SET) == -1){
-    tchdbsetecode(hdb, TCESEEK, __FILE__, __LINE__, __func__);
-    HDBUNLOCKFIO(hdb);
-    return false;
+  while(true){
+    int rb = pread(hdb->fd, buf, size, off);
+    if(rb >= size){
+      break;
+    } else if(rb > 0){
+      buf = (char *)buf + rb;
+      size -= rb;
+      off += rb;
+    } else if(rb == -1){
+      if(errno != EINTR){
+        tchdbsetecode(hdb, TCEREAD, __FILE__, __LINE__, __func__);
+        return false;
+      }
+    } else {
+      if(size > 0){
+        tchdbsetecode(hdb, TCEREAD, __FILE__, __LINE__, __func__);
+        return false;
+      }
+    }
   }
-  if(!tcread(hdb->fd, buf, size)){
-    tchdbsetecode(hdb, TCEREAD, __FILE__, __LINE__, __func__);
-    HDBUNLOCKFIO(hdb);
-    return false;
-  }
-  HDBUNLOCKFIO(hdb);
   return true;
 }
 
@@ -1114,8 +1182,7 @@ static void tcloadmeta(TCHDB *hdb, const char *hbuf){
 /* Get a natural prime number not less than a floor number.
    `num' specified the floor number.
    The return value is a prime number not less than the floor number. */
-static uint64_t tcgetprime(int num){
-  assert(num >= 0);
+static uint64_t tcgetprime(uint64_t num){
   uint64_t primes[] = {
     1, 2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 43, 47, 53, 59, 61, 71, 79, 83,
     89, 103, 109, 113, 127, 139, 157, 173, 191, 199, 223, 239, 251, 283, 317, 349,
@@ -1156,8 +1223,7 @@ static uint64_t tcgetprime(int num){
 static void tchdbclear(TCHDB *hdb){
   assert(hdb);
   hdb->mmtx = NULL;
-  hdb->cmtx = NULL;
-  hdb->fmtx = NULL;
+  hdb->dmtx = NULL;
   hdb->eckey = NULL;
   hdb->type = HDBTHASH;
   hdb->flags = 0;
@@ -1184,6 +1250,7 @@ static void tchdbclear(TCHDB *hdb){
   hdb->fbpool = NULL;
   hdb->fbpnum = 0;
   hdb->fbpmis = 0;
+  hdb->async = false;
   hdb->drpool = NULL;
   hdb->drpdef = NULL;
   hdb->drpoff = 0;
@@ -1205,6 +1272,7 @@ static void tchdbclear(TCHDB *hdb){
   hdb->cnt_appenddrp = -1;
   hdb->cnt_deferdrp = -1;
   hdb->cnt_flushdrp = -1;
+  hdb->cnt_adjrecc = -1;
   TCDODEBUG(hdb->cnt_writerec = 0);
   TCDODEBUG(hdb->cnt_reuserec = 0);
   TCDODEBUG(hdb->cnt_moverec = 0);
@@ -1218,6 +1286,7 @@ static void tchdbclear(TCHDB *hdb){
   TCDODEBUG(hdb->cnt_appenddrp = 0);
   TCDODEBUG(hdb->cnt_deferdrp = 0);
   TCDODEBUG(hdb->cnt_flushdrp = 0);
+  TCDODEBUG(hdb->cnt_adjrecc = 0);
 }
 
 
@@ -1847,18 +1916,18 @@ static int tcreckeycmp(const char *abuf, int asiz, const char *bbuf, int bsiz){
    The return value is true if successful, else, it is false. */
 static bool tchdbflushdrp(TCHDB *hdb){
   assert(hdb);
-  if(!HDBLOCKFIO(hdb)) return false;
+  if(!HDBLOCKDRP(hdb)) return false;
   if(!hdb->drpool){
-    HDBUNLOCKFIO(hdb);
+    HDBUNLOCKDRP(hdb);
     return true;
   }
   TCDODEBUG(hdb->cnt_flushdrp++);
-  if(!tcseekwrite(hdb, hdb->drpoff, tcxstrptr(hdb->drpool), tcxstrsize(hdb->drpool))){
-    HDBUNLOCKFIO(hdb);
+  if(!tcseekwrite(hdb, hdb->drpoff, TCXSTRPTR(hdb->drpool), TCXSTRSIZE(hdb->drpool))){
+    HDBUNLOCKDRP(hdb);
     return false;
   }
-  const char *rp = tcxstrptr(hdb->drpdef);
-  int size = tcxstrsize(hdb->drpdef);
+  const char *rp = TCXSTRPTR(hdb->drpdef);
+  int size = TCXSTRSIZE(hdb->drpdef);
   while(size > 0){
     int ksiz, vsiz;
     memcpy(&ksiz, rp, sizeof(int));
@@ -1875,7 +1944,7 @@ static bool tchdbflushdrp(TCHDB *hdb){
       hdb->drpool = NULL;
       hdb->drpdef = NULL;
       hdb->drpoff = 0;
-      HDBUNLOCKFIO(hdb);
+      HDBUNLOCKDRP(hdb);
       return false;
     }
     size -= sizeof(int) * 2 + ksiz + vsiz;
@@ -1892,7 +1961,7 @@ static bool tchdbflushdrp(TCHDB *hdb){
   llnum = hdb->fsiz;
   llnum = TCHTOILL(llnum);
   memcpy(hdb->map + HDBFSIZOFF, &llnum, sizeof(llnum));
-  HDBUNLOCKFIO(hdb);
+  HDBUNLOCKDRP(hdb);
   return true;
 }
 
@@ -1901,14 +1970,8 @@ static bool tchdbflushdrp(TCHDB *hdb){
    `hdb' specifies the hash tree database object. */
 static void tchdbcacheadjust(TCHDB *hdb){
   assert(hdb);
-  if(TCMAPRNUM(hdb->recc) < hdb->rcnum) return;
-  TCMAP *recc = hdb->recc;
-  tcmapiterinit(recc);
-  for(int i = 0; i < HDBCACHEOUT; i++){
-    int csiz;
-    const char *cbuf = tcmapiternext(recc, &csiz);
-    tcmapout(recc, cbuf, csiz);
-  }
+  TCDODEBUG(hdb->cnt_adjrecc++);
+  tcmdbcutfront(hdb->recc, HDBCACHEOUT);
 }
 
 
@@ -1987,10 +2050,14 @@ static bool tchdbopenimpl(TCHDB *hdb, const char *path, int omode){
     }
     sbuf.st_size = hdb->fsiz;
   }
-  hdb->fd = fd;
-  if(!tcseekread(hdb, 0, hbuf, HDBHEADSIZ)){
+  if(lseek(fd, 0, SEEK_SET) == -1){
+    tchdbsetecode(hdb, TCESEEK, __FILE__, __LINE__, __func__);
+    close(fd);
+    return false;
+  }
+  if(!tcread(fd, hbuf, HDBHEADSIZ)){
+    tchdbsetecode(hdb, TCEREAD, __FILE__, __LINE__, __func__);
     close(fd);
-    hdb->fd = -1;
     return false;
   }
   int type = hdb->type;
@@ -2002,7 +2069,6 @@ static bool tchdbopenimpl(TCHDB *hdb, const char *path, int omode){
        hdb->frec < HDBHEADSIZ + fbpsiz || hdb->frec > hdb->fsiz || sbuf.st_size < hdb->fsiz){
       tchdbsetecode(hdb, TCEMETA, __FILE__, __LINE__, __func__);
       close(fd);
-      hdb->fd = -1;
       return false;
     }
     if(sbuf.st_size > hdb->fsiz) hdb->fsiz = sbuf.st_size;
@@ -2010,7 +2076,6 @@ static bool tchdbopenimpl(TCHDB *hdb, const char *path, int omode){
   if((hdb->opts & HDBTDEFLATE) && !_tc_deflate){
     tchdbsetecode(hdb, TCEINVALID, __FILE__, __LINE__, __func__);
     close(fd);
-    hdb->fd = -1;
     return false;
   }
   int besiz = (hdb->opts & HDBTLARGE) ? sizeof(int64_t) : sizeof(int32_t);
@@ -2020,7 +2085,6 @@ static bool tchdbopenimpl(TCHDB *hdb, const char *path, int omode){
   if(map == MAP_FAILED){
     tchdbsetecode(hdb, TCEMMAP, __FILE__, __LINE__, __func__);
     close(fd);
-    hdb->fd = -1;
     return false;
   }
   hdb->fbpmax = fbpmax;
@@ -2032,10 +2096,11 @@ static bool tchdbopenimpl(TCHDB *hdb, const char *path, int omode){
   }
   hdb->fbpnum = 0;
   hdb->fbpmis = 0;
+  hdb->async = false;
   hdb->drpool = NULL;
   hdb->drpdef = NULL;
   hdb->drpoff = 0;
-  hdb->recc = (hdb->rcnum > 0) ? tcmapnew2(hdb->rcnum * 2 + 1) : NULL;
+  hdb->recc = (hdb->rcnum > 0) ? tcmdbnew2(hdb->rcnum * 2 + 1) : NULL;
   hdb->path = tcstrdup(path);
   hdb->omode = omode;
   hdb->iter = 0;
@@ -2053,6 +2118,7 @@ static bool tchdbopenimpl(TCHDB *hdb, const char *path, int omode){
   hdb->zmode = (hdb->opts & HDBTDEFLATE) || (hdb->opts & HDBTTCBS);
   hdb->ecode = TCESUCCESS;
   hdb->fatal = false;
+  hdb->fd = fd;
   if(hdb->omode & HDBOWRITER){
     bool err = false;
     if(!(hdb->flags & HDBFOPEN) && !tchdbloadfbp(hdb)) err = true;
@@ -2081,7 +2147,7 @@ static bool tchdbcloseimpl(TCHDB *hdb){
   assert(hdb);
   bool err = false;
   if(hdb->recc){
-    tcmapdel(hdb->recc);
+    tcmdbdel(hdb->recc);
     hdb->recc = NULL;
   }
   if(hdb->omode & HDBOWRITER){
@@ -2102,7 +2168,7 @@ static bool tchdbcloseimpl(TCHDB *hdb){
   }
   hdb->path = NULL;
   hdb->fd = -1;
-  return err ? false : true;
+  return !err;
 }
 
 
@@ -2117,11 +2183,7 @@ static bool tchdbcloseimpl(TCHDB *hdb){
 static bool tchdbputimpl(TCHDB *hdb, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
                          int dmode){
   assert(hdb && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
-  if(hdb->recc){
-    bool clk = HDBLOCKCACHE(hdb, true);
-    tcmapout(hdb->recc, kbuf, ksiz);
-    if(clk) HDBUNLOCKCACHE(hdb);
-  }
+  if(hdb->recc) tcmdbout(hdb->recc, kbuf, ksiz);
   uint8_t hash;
   uint64_t bidx = tchdbbidx(hdb, kbuf, ksiz, &hash);
   off_t off = tchdbgetbucket(hdb, bidx);
@@ -2272,11 +2334,7 @@ static void tchdbdrpappend(TCHDB *hdb, const char *kbuf, int ksiz, const char *v
    If successful, the return value is true, else, it is false. */
 static bool tchdbputasyncimpl(TCHDB *hdb, const char *kbuf, int ksiz, const char *vbuf, int vsiz){
   assert(hdb && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
-  if(hdb->recc){
-    bool clk = HDBLOCKCACHE(hdb, true);
-    tcmapout(hdb->recc, kbuf, ksiz);
-    if(clk) HDBUNLOCKCACHE(hdb);
-  }
+  if(hdb->recc) tcmdbout(hdb->recc, kbuf, ksiz);
   if(!hdb->drpool){
     hdb->drpool = tcxstrnew3(HDBDRPUNIT + HDBDRPLAT);
     hdb->drpdef = tcxstrnew3(HDBDRPUNIT);
@@ -2296,6 +2354,7 @@ static bool tchdbputasyncimpl(TCHDB *hdb, const char *kbuf, int ksiz, const char
       TCXSTRCAT(drpdef, &vsiz, sizeof(vsiz));
       TCXSTRCAT(drpdef, kbuf, ksiz);
       TCXSTRCAT(drpdef, vbuf, vsiz);
+      if(TCXSTRSIZE(hdb->drpdef) > HDBDRPUNIT && !tchdbflushdrp(hdb)) return false;
       return true;
     }
     rec.off = off;
@@ -2314,6 +2373,7 @@ static bool tchdbputasyncimpl(TCHDB *hdb, const char *kbuf, int ksiz, const char
       TCXSTRCAT(drpdef, &vsiz, sizeof(vsiz));
       TCXSTRCAT(drpdef, kbuf, ksiz);
       TCXSTRCAT(drpdef, vbuf, vsiz);
+      if(TCXSTRSIZE(hdb->drpdef) > HDBDRPUNIT && !tchdbflushdrp(hdb)) return false;
       return true;
     }
   }
@@ -2332,7 +2392,7 @@ static bool tchdbputasyncimpl(TCHDB *hdb, const char *kbuf, int ksiz, const char
   }
   tchdbdrpappend(hdb, kbuf, ksiz, vbuf, vsiz, hash);
   hdb->rnum++;
-  if(tcxstrsize(hdb->drpool) > HDBDRPUNIT && !tchdbflushdrp(hdb)) return false;
+  if(TCXSTRSIZE(hdb->drpool) > HDBDRPUNIT && !tchdbflushdrp(hdb)) return false;
   return true;
 }
 
@@ -2344,11 +2404,7 @@ static bool tchdbputasyncimpl(TCHDB *hdb, const char *kbuf, int ksiz, const char
    If successful, the return value is true, else, it is false. */
 static bool tchdboutimpl(TCHDB *hdb, const char *kbuf, int ksiz){
   assert(hdb && kbuf && ksiz >= 0);
-  if(hdb->recc){
-    bool clk = HDBLOCKCACHE(hdb, true);
-    tcmapout(hdb->recc, kbuf, ksiz);
-    if(clk) HDBUNLOCKCACHE(hdb);
-  }
+  if(hdb->recc) tcmdbout(hdb->recc, kbuf, ksiz);
   uint8_t hash;
   uint64_t bidx = tchdbbidx(hdb, kbuf, ksiz, &hash);
   off_t off = tchdbgetbucket(hdb, bidx);
@@ -2450,24 +2506,18 @@ static bool tchdboutimpl(TCHDB *hdb, const char *kbuf, int ksiz){
 static char *tchdbgetimpl(TCHDB *hdb, const char *kbuf, int ksiz, int *sp){
   assert(hdb && kbuf && ksiz >= 0 && sp);
   if(hdb->recc){
-    bool clk = HDBLOCKCACHE(hdb, false);
     int tvsiz;
-    const char *tvbuf = tcmapget(hdb->recc, kbuf, ksiz, &tvsiz);
+    char *tvbuf = tcmdbget(hdb->recc, kbuf, ksiz, &tvsiz);
     if(tvbuf){
       if(*tvbuf == '*'){
-        if(clk) HDBUNLOCKCACHE(hdb);
         tchdbsetecode(hdb, TCENOREC, __FILE__, __LINE__, __func__);
+        free(tvbuf);
         return NULL;
       }
-      tvbuf++;
-      tvsiz--;
-      *sp = tvsiz;
-      char *rv;
-      TCMEMDUP(rv, tvbuf, tvsiz);
-      if(clk) HDBUNLOCKCACHE(hdb);
-      return rv;
+      *sp = tvsiz - 1;
+      memmove(tvbuf, tvbuf + 1, tvsiz);
+      return tvbuf;
     }
-    if(clk) HDBUNLOCKCACHE(hdb);
   }
   uint8_t hash;
   uint64_t bidx = tchdbbidx(hdb, kbuf, ksiz, &hash);
@@ -2510,10 +2560,8 @@ static char *tchdbgetimpl(TCHDB *hdb, const char *kbuf, int ksiz, int *sp){
             return NULL;
           }
           if(hdb->recc){
-            bool clk = HDBLOCKCACHE(hdb, true);
-            tchdbcacheadjust(hdb);
-            tcmapput3(hdb->recc, kbuf, ksiz, "=", 1, zbuf, zsiz);
-            if(clk) HDBUNLOCKCACHE(hdb);
+            if(tcmdbrnum(hdb->recc) >= hdb->rcnum) tchdbcacheadjust(hdb);
+            tcmdbput3(hdb->recc, kbuf, ksiz, "=", 1, zbuf, zsiz);
           }
           *sp = zsiz;
           return zbuf;
@@ -2525,10 +2573,8 @@ static char *tchdbgetimpl(TCHDB *hdb, const char *kbuf, int ksiz, int *sp){
           return rec.bbuf;
         }
         if(hdb->recc){
-          bool clk = HDBLOCKCACHE(hdb, true);
-          tchdbcacheadjust(hdb);
-          tcmapput3(hdb->recc, kbuf, ksiz, "=", 1, rec.vbuf, rec.vsiz);
-          if(clk) HDBUNLOCKCACHE(hdb);
+          if(tcmdbrnum(hdb->recc) >= hdb->rcnum) tchdbcacheadjust(hdb);
+          tcmdbput3(hdb->recc, kbuf, ksiz, "=", 1, rec.vbuf, rec.vsiz);
         }
         *sp = rec.vsiz;
         char *rv;
@@ -2538,10 +2584,8 @@ static char *tchdbgetimpl(TCHDB *hdb, const char *kbuf, int ksiz, int *sp){
     }
   }
   if(hdb->recc){
-    bool clk = HDBLOCKCACHE(hdb, true);
-    tchdbcacheadjust(hdb);
-    tcmapput(hdb->recc, kbuf, ksiz, "*", 1);
-    if(clk) HDBUNLOCKCACHE(hdb);
+    if(tcmdbrnum(hdb->recc) >= hdb->rcnum) tchdbcacheadjust(hdb);
+    tcmdbput(hdb->recc, kbuf, ksiz, "*", 1);
   }
   tchdbsetecode(hdb, TCENOREC, __FILE__, __LINE__, __func__);
   return NULL;
@@ -2559,23 +2603,19 @@ static char *tchdbgetimpl(TCHDB *hdb, const char *kbuf, int ksiz, int *sp){
 static int tchdbgetintobuf(TCHDB *hdb, const char *kbuf, int ksiz, char *vbuf, int max){
   assert(hdb && kbuf && ksiz >= 0 && vbuf && max >= 0);
   if(hdb->recc){
-    bool clk = HDBLOCKCACHE(hdb, false);
     int tvsiz;
-    const char *tvbuf = tcmapget(hdb->recc, kbuf, ksiz, &tvsiz);
+    char *tvbuf = tcmdbget(hdb->recc, kbuf, ksiz, &tvsiz);
     if(tvbuf){
       if(*tvbuf == '*'){
-        if(clk) HDBUNLOCKCACHE(hdb);
         tchdbsetecode(hdb, TCENOREC, __FILE__, __LINE__, __func__);
+        free(tvbuf);
         return -1;
       }
-      tvbuf++;
-      tvsiz--;
-      tvsiz = tclmin(tvsiz, max);
-      memcpy(vbuf, tvbuf, tvsiz);
-      if(clk) HDBUNLOCKCACHE(hdb);
+      tvsiz = tclmin(tvsiz - 1, max);
+      memcpy(vbuf, tvbuf + 1, tvsiz);
+      free(tvbuf);
       return tvsiz;
     }
-    if(clk) HDBUNLOCKCACHE(hdb);
   }
   uint8_t hash;
   uint64_t bidx = tchdbbidx(hdb, kbuf, ksiz, &hash);
@@ -2618,10 +2658,8 @@ static int tchdbgetintobuf(TCHDB *hdb, const char *kbuf, int ksiz, char *vbuf, i
             return -1;
           }
           if(hdb->recc){
-            bool clk = HDBLOCKCACHE(hdb, true);
-            tchdbcacheadjust(hdb);
-            tcmapput3(hdb->recc, kbuf, ksiz, "=", 1, zbuf, zsiz);
-            if(clk) HDBUNLOCKCACHE(hdb);
+            if(tcmdbrnum(hdb->recc) >= hdb->rcnum) tchdbcacheadjust(hdb);
+            tcmdbput3(hdb->recc, kbuf, ksiz, "=", 1, zbuf, zsiz);
           }
           zsiz = tclmin(zsiz, max);
           memcpy(vbuf, zbuf, zsiz);
@@ -2629,10 +2667,8 @@ static int tchdbgetintobuf(TCHDB *hdb, const char *kbuf, int ksiz, char *vbuf, i
           return zsiz;
         }
         if(hdb->recc){
-          bool clk = HDBLOCKCACHE(hdb, true);
-          tchdbcacheadjust(hdb);
-          tcmapput3(hdb->recc, kbuf, ksiz, "=", 1, rec.vbuf, rec.vsiz);
-          if(clk) HDBUNLOCKCACHE(hdb);
+          if(tcmdbrnum(hdb->recc) >= hdb->rcnum) tchdbcacheadjust(hdb);
+          tcmdbput3(hdb->recc, kbuf, ksiz, "=", 1, rec.vbuf, rec.vsiz);
         }
         int vsiz = tclmin(rec.vsiz, max);
         memcpy(vbuf, rec.vbuf, vsiz);
@@ -2642,10 +2678,8 @@ static int tchdbgetintobuf(TCHDB *hdb, const char *kbuf, int ksiz, char *vbuf, i
     }
   }
   if(hdb->recc){
-    bool clk = HDBLOCKCACHE(hdb, true);
-    tchdbcacheadjust(hdb);
-    tcmapput(hdb->recc, kbuf, ksiz, "*", 1);
-    if(clk) HDBUNLOCKCACHE(hdb);
+    if(tcmdbrnum(hdb->recc) >= hdb->rcnum) tchdbcacheadjust(hdb);
+    tcmdbput(hdb->recc, kbuf, ksiz, "*", 1);
   }
   tchdbsetecode(hdb, TCENOREC, __FILE__, __LINE__, __func__);
   return -1;
@@ -2661,19 +2695,17 @@ static int tchdbgetintobuf(TCHDB *hdb, const char *kbuf, int ksiz, char *vbuf, i
 static int tchdbvsizimpl(TCHDB *hdb, const char *kbuf, int ksiz){
   assert(hdb && kbuf && ksiz >= 0);
   if(hdb->recc){
-    bool clk = HDBLOCKCACHE(hdb, false);
     int tvsiz;
-    const char *tvbuf = tcmapget(hdb->recc, kbuf, ksiz, &tvsiz);
+    char *tvbuf = tcmdbget(hdb->recc, kbuf, ksiz, &tvsiz);
     if(tvbuf){
       if(*tvbuf == '*'){
-        if(clk) HDBUNLOCKCACHE(hdb);
         tchdbsetecode(hdb, TCENOREC, __FILE__, __LINE__, __func__);
+        free(tvbuf);
         return -1;
       }
-      if(clk) HDBUNLOCKCACHE(hdb);
+      free(tvbuf);
       return tvsiz - 1;
     }
-    if(clk) HDBUNLOCKCACHE(hdb);
   }
   uint8_t hash;
   uint64_t bidx = tchdbbidx(hdb, kbuf, ksiz, &hash);
@@ -2716,19 +2748,15 @@ static int tchdbvsizimpl(TCHDB *hdb, const char *kbuf, int ksiz){
             return -1;
           }
           if(hdb->recc){
-            bool clk = HDBLOCKCACHE(hdb, true);
-            tchdbcacheadjust(hdb);
-            tcmapput3(hdb->recc, kbuf, ksiz, "=", 1, zbuf, zsiz);
-            if(clk) HDBUNLOCKCACHE(hdb);
+            if(tcmdbrnum(hdb->recc) >= hdb->rcnum) tchdbcacheadjust(hdb);
+            tcmdbput3(hdb->recc, kbuf, ksiz, "=", 1, zbuf, zsiz);
           }
           free(zbuf);
           return zsiz;
         }
         if(hdb->recc && rec.vbuf){
-          bool clk = HDBLOCKCACHE(hdb, true);
-          tchdbcacheadjust(hdb);
-          tcmapput3(hdb->recc, kbuf, ksiz, "=", 1, rec.vbuf, rec.vsiz);
-          if(clk) HDBUNLOCKCACHE(hdb);
+          if(tcmdbrnum(hdb->recc) >= hdb->rcnum) tchdbcacheadjust(hdb);
+          tcmdbput3(hdb->recc, kbuf, ksiz, "=", 1, rec.vbuf, rec.vsiz);
         }
         free(rec.bbuf);
         return rec.vsiz;
@@ -2736,10 +2764,8 @@ static int tchdbvsizimpl(TCHDB *hdb, const char *kbuf, int ksiz){
     }
   }
   if(hdb->recc){
-    bool clk = HDBLOCKCACHE(hdb, true);
-    tchdbcacheadjust(hdb);
-    tcmapput(hdb->recc, kbuf, ksiz, "*", 1);
-    if(clk) HDBUNLOCKCACHE(hdb);
+    if(tcmdbrnum(hdb->recc) >= hdb->rcnum) tchdbcacheadjust(hdb);
+    tcmdbput(hdb->recc, kbuf, ksiz, "*", 1);
   }
   tchdbsetecode(hdb, TCENOREC, __FILE__, __LINE__, __func__);
   return -1;
@@ -2859,7 +2885,7 @@ static bool tchdboptimizeimpl(TCHDB *hdb, int64_t bnum, int8_t apow, int8_t fpow
   TCXSTR *key = tcxstrnew();
   TCXSTR *val = tcxstrnew();
   while(!err && tchdbiternextintoxstr(hdb, key, val)){
-    if(!tchdbputkeep(thdb, tcxstrptr(key), tcxstrsize(key), tcxstrptr(val), tcxstrsize(val))){
+    if(!tchdbputkeep(thdb, TCXSTRPTR(key), TCXSTRSIZE(key), TCXSTRPTR(val), TCXSTRSIZE(val))){
       tchdbsetecode(hdb, thdb->ecode, __FILE__, __LINE__, __func__);
       err = true;
     }
@@ -2904,7 +2930,7 @@ static bool tchdbvanishimpl(TCHDB *hdb){
   if(!tchdbcloseimpl(hdb)) err = true;
   if(!tchdbopenimpl(hdb, path, HDBOTRUNC | omode)) err = true;
   free(path);
-  return err ? false : true;
+  return !err;
 }
 
 
@@ -2937,13 +2963,12 @@ static bool tchdbunlockmethod(TCHDB *hdb){
 }
 
 
-/* Lock the cache of the hash database object.
+/* Lock the delayed record pool of the hash database object.
    `hdb' specifies the hash database object.
-   `wr' specifies whether the lock is writer or not.
    If successful, the return value is true, else, it is false. */
-static bool tchdblockcache(TCHDB *hdb, bool wr){
+static bool tchdblockdrp(TCHDB *hdb){
   assert(hdb);
-  if(wr ? pthread_rwlock_wrlock(hdb->cmtx) != 0 : pthread_rwlock_rdlock(hdb->cmtx) != 0){
+  if(pthread_mutex_lock(hdb->dmtx) != 0){
     tchdbsetecode(hdb, TCETHREAD, __FILE__, __LINE__, __func__);
     return false;
   }
@@ -2952,40 +2977,12 @@ static bool tchdblockcache(TCHDB *hdb, bool wr){
 }
 
 
-/* Unlock the cache of the hash database object.
+/* Unlock the delayed record pool of the hash database object.
    `hdb' specifies the hash database object.
    If successful, the return value is true, else, it is false. */
-static bool tchdbunlockcache(TCHDB *hdb){
+static bool tchdbunlockdrp(TCHDB *hdb){
   assert(hdb);
-  if(pthread_rwlock_unlock(hdb->cmtx) != 0){
-    tchdbsetecode(hdb, TCETHREAD, __FILE__, __LINE__, __func__);
-    return false;
-  }
-  TCTESTYIELD();
-  return true;
-}
-
-
-/* Lock the file I/O of the hash database object.
-   `hdb' specifies the hash database object.
-   If successful, the return value is true, else, it is false. */
-static bool tchdblockfio(TCHDB *hdb){
-  assert(hdb);
-  if(pthread_mutex_lock(hdb->fmtx) != 0){
-    tchdbsetecode(hdb, TCETHREAD, __FILE__, __LINE__, __func__);
-    return false;
-  }
-  TCTESTYIELD();
-  return true;
-}
-
-
-/* Unlock the file I/O of the hash database object.
-   `hdb' specifies the hash database object.
-   If successful, the return value is true, else, it is false. */
-static bool tchdbunlockfio(TCHDB *hdb){
-  assert(hdb);
-  if(pthread_mutex_unlock(hdb->fmtx) != 0){
+  if(pthread_mutex_unlock(hdb->dmtx) != 0){
     tchdbsetecode(hdb, TCETHREAD, __FILE__, __LINE__, __func__);
     return false;
   }
@@ -3009,8 +3006,7 @@ void tchdbprintmeta(TCHDB *hdb){
   char *wp = buf;
   wp += sprintf(wp, "META:");
   wp += sprintf(wp, " mmtx=%p", (void *)hdb->mmtx);
-  wp += sprintf(wp, " cmtx=%p", (void *)hdb->cmtx);
-  wp += sprintf(wp, " fmtx=%p", (void *)hdb->fmtx);
+  wp += sprintf(wp, " dmtx=%p", (void *)hdb->dmtx);
   wp += sprintf(wp, " eckey=%p", (void *)hdb->eckey);
   wp += sprintf(wp, " type=%02X", hdb->type);
   wp += sprintf(wp, " flags=%02X", hdb->flags);
@@ -3040,6 +3036,8 @@ void tchdbprintmeta(TCHDB *hdb){
   wp += sprintf(wp, " drpool=%p", (void *)hdb->drpool);
   wp += sprintf(wp, " drpdef=%p", (void *)hdb->drpdef);
   wp += sprintf(wp, " drpoff=%llu", (unsigned long long)hdb->drpoff);
+  wp += sprintf(wp, " recc=%p", (void *)hdb->recc);
+  wp += sprintf(wp, " rcnum=%u", hdb->rcnum);
   wp += sprintf(wp, " ecode=%d", hdb->ecode);
   wp += sprintf(wp, " fatal=%u", hdb->fatal);
   wp += sprintf(wp, " inode=%llu", (unsigned long long)(uint64_t)hdb->inode);
@@ -3058,6 +3056,7 @@ void tchdbprintmeta(TCHDB *hdb){
   wp += sprintf(wp, " cnt_appenddrp=%lld", (long long)hdb->cnt_appenddrp);
   wp += sprintf(wp, " cnt_deferdrp=%lld", (long long)hdb->cnt_deferdrp);
   wp += sprintf(wp, " cnt_flushdrp=%lld", (long long)hdb->cnt_flushdrp);
+  wp += sprintf(wp, " cnt_adjrecc=%lld", (long long)hdb->cnt_adjrecc);
   *(wp++) = '\n';
   tcwrite(hdb->dbgfd, buf, wp - buf);
 }
