@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The hash database API of Tokyo Cabinet
- *                                                      Copyright (C) 2006-2007 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -28,9 +28,9 @@ __TCHDB_CLINKAGEBEGIN
 
 
 #include <stdlib.h>
-#include <time.h>
 #include <stdbool.h>
 #include <stdint.h>
+#include <time.h>
 #include <tcutil.h>
 
 
@@ -42,8 +42,7 @@ __TCHDB_CLINKAGEBEGIN
 
 typedef struct {                         /* type of structure for a hash database */
   void *mmtx;                            /* mutex for method */
-  void *cmtx;                            /* mutex for cache */
-  void *fmtx;                            /* mutex for file I/O */
+  void *dmtx;                            /* mutex for DRP */
   void *eckey;                           /* key for thread specific error code */
   uint8_t type;                          /* database type */
   uint8_t flags;                         /* additional flags */
@@ -71,11 +70,12 @@ typedef struct {                         /* type of structure for a hash databas
   void *fbpool;                          /* free block pool */
   int32_t fbpnum;                        /* number of the free block pool */
   int32_t fbpmis;                        /* number of missing retrieval of the free block pool */
+  bool async;                            /* whether asynchronous storing is called */
   TCXSTR *drpool;                        /* delayed record pool */
   TCXSTR *drpdef;                        /* deferred records of the delayed record pool */
   uint64_t drpoff;                       /* offset of the delayed record pool */
-  TCMAP *recc;                           /* cache for records */
-  int rcnum;                             /* max number of cached records */
+  TCMDB *recc;                           /* cache for records */
+  uint32_t rcnum;                        /* max number of cached records */
   int ecode;                             /* last happened error code */
   bool fatal;                            /* whether a fatal error occured */
   uint64_t inode;                        /* inode number */
@@ -94,6 +94,7 @@ typedef struct {                         /* type of structure for a hash databas
   int64_t cnt_appenddrp;                 /* tesing counter for DRP append times */
   int64_t cnt_deferdrp;                  /* tesing counter for DRP defer times */
   int64_t cnt_flushdrp;                  /* tesing counter for DRP flush times */
+  int64_t cnt_adjrecc;                   /* tesing counter for record cache adjust times */
 } TCHDB;
 
 enum {                                   /* enumeration for error codes */
@@ -354,9 +355,9 @@ void *tchdbget(TCHDB *hdb, const void *kbuf, int ksiz, int *sp);
    `hdb' specifies the hash database object.
    `kstr' specifies the string of the key.
    If successful, the return value is the string of the value of the corresponding record.
-   `NULL' is returned if no record corresponds.  Because the region of the return value is
-   allocated with the `malloc' call, it should be released with the `free' call when it is no
-   longer in use. */
+   `NULL' is returned if no record corresponds.
+   Because the region of the return value is allocated with the `malloc' call, it should be
+   released with the `free' call when it is no longer in use. */
 char *tchdbget2(TCHDB *hdb, const char *kstr);
 
 
@@ -437,6 +438,33 @@ char *tchdbiternext2(TCHDB *hdb);
 bool tchdbiternext3(TCHDB *hdb, TCXSTR *kxstr, TCXSTR *vxstr);
 
 
+/* Get forward matching keys in a hash database object.
+   `hdb' specifies the hash database object.
+   `pbuf' specifies the pointer to the region of the prefix.
+   `psiz' specifies the size of the region of the prefix.
+   `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
+   specified.
+   The return value is a list object of the corresponding keys.  This function does never fail
+   and return an empty list even if no key corresponds.
+   Because the object of the return value is created with the function `tclistnew', it should be
+   deleted with the function `tclistdel' when it is no longer in use.  Note that this function
+   may be very slow because every key in the database is scanned. */
+TCLIST *tchdbfwmkeys(TCHDB *hdb, const void *pbuf, int psiz, int max);
+
+
+/* Get forward matching string keys in a hash database object.
+   `hdb' specifies the hash database object.
+   `pstr' specifies the string of the prefix.
+   `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
+   specified.
+   The return value is a list object of the corresponding keys.  This function does never fail
+   and return an empty list even if no key corresponds.
+   Because the object of the return value is created with the function `tclistnew', it should be
+   deleted with the function `tclistdel' when it is no longer in use.  Note that this function
+   may be very slow because every key in the database is scanned. */
+TCLIST *tchdbfwmkeys2(TCHDB *hdb, const char *pstr, int max);
+
+
 /* Synchronize updated contents of a hash database object with the file and the device.
    `hdb' specifies the hash database object connected as a writer.
    If successful, the return value is true, else, it is false.
