@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The command line utility of the hash database API
- *                                                      Copyright (C) 2006-2007 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -18,8 +18,6 @@
 #include <tchdb.h>
 #include "myconf.h"
 
-#define RECBUFSIZ      32                // buffer for records
-
 
 /* global variables */
 const char *g_progname;                  // program name
@@ -48,7 +46,7 @@ static int procput(const char *path, const char *kbuf, int ksiz, const char *vbu
                    int omode, int dmode);
 static int procout(const char *path, const char *kbuf, int ksiz, int omode);
 static int procget(const char *path, const char *kbuf, int ksiz, int omode, bool px, bool pz);
-static int proclist(const char *path, int omode, int max, bool pv, bool px);
+static int proclist(const char *path, int omode, int max, bool pv, bool px, const char *fmstr);
 static int procoptimize(const char *path, int bnum, int apow, int fpow, int opts, int omode);
 static int procimporttsv(const char *path, const char *file, int omode, bool sc);
 static int procversion(void);
@@ -97,7 +95,7 @@ static void usage(void){
   fprintf(stderr, "  %s put [-nl|-nb] [-sx] [-dk|-dc] path key value\n", g_progname);
   fprintf(stderr, "  %s out [-nl|-nb] [-sx] path key\n", g_progname);
   fprintf(stderr, "  %s get [-nl|-nb] [-sx] [-px] [-pz] path key\n", g_progname);
-  fprintf(stderr, "  %s list [-nl|-nb] [-m num] [-pv] [-px] path\n", g_progname);
+  fprintf(stderr, "  %s list [-nl|-nb] [-m num] [-pv] [-px] [-fm str] path\n", g_progname);
   fprintf(stderr, "  %s optimize [-tl] [-td|-tb] [-tz] [-nl|-nb] path [bnum [apow [fpow]]]\n",
           g_progname);
   fprintf(stderr, "  %s importtsv [-nl|-nb] [-sc] path [file]\n", g_progname);
@@ -382,6 +380,7 @@ static int runlist(int argc, char **argv){
   int max = -1;
   bool pv = false;
   bool px = false;
+  char *fmstr = NULL;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-nl")){
@@ -395,6 +394,9 @@ static int runlist(int argc, char **argv){
         pv = true;
       } else if(!strcmp(argv[i], "-px")){
         px = true;
+      } else if(!strcmp(argv[i], "-fm")){
+        if(++i >= argc) usage();
+        fmstr = argv[i];
       } else {
         usage();
       }
@@ -405,7 +407,7 @@ static int runlist(int argc, char **argv){
     }
   }
   if(!path) usage();
-  int rv = proclist(path, omode, max, pv, px);
+  int rv = proclist(path, omode, max, pv, px, fmstr);
   return rv;
 }
 
@@ -550,6 +552,10 @@ static int procinform(const char *path, int omode){
     printf("used bucket number: %lld\n", (long long)tchdbbnumused(hdb));
   printf("alignment: %u\n", tchdbalign(hdb));
   printf("free block pool: %u\n", tchdbfbpmax(hdb));
+  printf("inode number: %lld\n", (long long)tchdbinode(hdb));
+  char date[48];
+  tcdatestrwww(tchdbmtime(hdb), INT_MAX, date);
+  printf("modified time: %s\n", date);
   uint8_t opts = tchdbopts(hdb);
   printf("options:");
   if(opts & HDBTLARGE) printf(" large");
@@ -660,7 +666,7 @@ static int procget(const char *path, const char *kbuf, int ksiz, int omode, bool
 
 
 /* perform list command */
-static int proclist(const char *path, int omode, int max, bool pv, bool px){
+static int proclist(const char *path, int omode, int max, bool pv, bool px, const char *fmstr){
   TCHDB *hdb = tchdbnew();
   if(g_dbgfd >= 0) tchdbsetdbgfd(hdb, g_dbgfd);
   if(!tchdbopen(hdb, path, HDBOREADER | omode)){
@@ -669,24 +675,44 @@ static int proclist(const char *path, int omode, int max, bool pv, bool px){
     return 1;
   }
   bool err = false;
-  if(!tchdbiterinit(hdb)){
-    printerr(hdb);
-    err = true;
-  }
-  TCXSTR *key = tcxstrnew();
-  TCXSTR *val = tcxstrnew();
-  int cnt = 0;
-  while(tchdbiternext3(hdb, key, val)){
-    printdata(tcxstrptr(key), tcxstrsize(key), px);
-    if(pv){
-      putchar('\t');
-      printdata(tcxstrptr(val), tcxstrsize(val), px);
+  if(fmstr){
+    TCLIST *keys = tchdbfwmkeys2(hdb, fmstr, max);
+    for(int i = 0; i < tclistnum(keys); i++){
+      int ksiz;
+      const char *kbuf = tclistval(keys, i, &ksiz);
+      printdata(kbuf, ksiz, px);
+      if(pv){
+        int vsiz;
+        char *vbuf = tchdbget(hdb, kbuf, ksiz, &vsiz);
+        if(vbuf){
+          putchar('\t');
+          printdata(vbuf, vsiz, px);
+          free(vbuf);
+        }
+      }
+      putchar('\n');
     }
-    putchar('\n');
-    if(max >= 0 && ++cnt >= max) break;
+    tclistdel(keys);
+  } else {
+    if(!tchdbiterinit(hdb)){
+      printerr(hdb);
+      err = true;
+    }
+    TCXSTR *key = tcxstrnew();
+    TCXSTR *val = tcxstrnew();
+    int cnt = 0;
+    while(tchdbiternext3(hdb, key, val)){
+      printdata(tcxstrptr(key), tcxstrsize(key), px);
+      if(pv){
+        putchar('\t');
+        printdata(tcxstrptr(val), tcxstrsize(val), px);
+      }
+      putchar('\n');
+      if(max >= 0 && ++cnt >= max) break;
+    }
+    tcxstrdel(val);
+    tcxstrdel(key);
   }
-  tcxstrdel(val);
-  tcxstrdel(key);
   if(!tchdbclose(hdb)){
     if(!err) printerr(hdb);
     err = true;
@@ -768,7 +794,7 @@ static int procimporttsv(const char *path, const char *file, int omode, bool sc)
 /* perform version command */
 static int procversion(void){
   printf("Tokyo Cabinet version %s (%d:%s)\n", tcversion, _TC_LIBVER, _TC_FORMATVER);
-  printf("Copyright (C) 2006-2007 Mikio Hirabayashi\n");
+  printf("Copyright (C) 2006-2008 Mikio Hirabayashi\n");
   return 0;
 }
 
