@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The test cases of the hash database API
- *                                                      Copyright (C) 2006-2007 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -55,6 +55,7 @@ typedef struct {                         // type of structure for typical thread
   TCHDB *hdb;
   int rnum;
   bool nc;
+  int rratio;
   int id;
 } TARGTYPICAL;
 
@@ -71,6 +72,7 @@ static void iprintf(const char *format, ...);
 static void eprint(TCHDB *hdb, const char *func);
 static void mprint(TCHDB *hdb);
 static int myrand(int range);
+static int myrandnd(int range);
 static int runwrite(int argc, char **argv);
 static int runread(int argc, char **argv);
 static int runremove(int argc, char **argv);
@@ -78,11 +80,11 @@ static int runwicked(int argc, char **argv);
 static int runtypical(int argc, char **argv);
 static int procwrite(const char *path, int tnum, int rnum, int bnum, int apow, int fpow,
                      int opts, int omode, bool as, bool rnd);
-static int procread(const char *path, int tnum, int omode, bool wb, bool rnd);
+static int procread(const char *path, int tnum, int rcnum, int omode, bool wb, bool rnd);
 static int procremove(const char *path, int tnum, int omode, bool rnd);
 static int procwicked(const char *path, int tnum, int rnum, int opts, int omode, bool nc);
 static int proctypical(const char *path, int tnum, int rnum, int bnum, int apow, int fpow,
-                       int opts, int omode, bool nc);
+                       int opts, int rcnum, int omode, bool nc, int rratio);
 static void *threadwrite(void *targ);
 static void *threadread(void *targ);
 static void *threadremove(void *targ);
@@ -96,7 +98,7 @@ int main(int argc, char **argv){
   g_dbgfd = -1;
   const char *ebuf = getenv("TCDBGFD");
   if(ebuf) g_dbgfd = atoi(ebuf);
-  srand((unsigned int)(tctime() * 100) % UINT_MAX);
+  srand((unsigned int)(tctime() * 1000) % UINT_MAX);
   if(argc < 2) usage();
   int rv = 0;
   if(!strcmp(argv[1], "write")){
@@ -123,12 +125,12 @@ static void usage(void){
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s write [-tl] [-td|-tb] [-nl|-nb] [-as] [-rnd] path tnum rnum"
           " [bnum [apow [fpow]]]\n", g_progname);
-  fprintf(stderr, "  %s read [-nl|-nb] [-wb] [-rnd] path tnum\n", g_progname);
+  fprintf(stderr, "  %s read [-rc num] [-nl|-nb] [-wb] [-rnd] path tnum\n", g_progname);
   fprintf(stderr, "  %s remove [-nl|-nb] [-wb] [-rnd] path tnum\n", g_progname);
   fprintf(stderr, "  %s wicked [-tl] [-td|-tb] [-nl|-nb] [-nc] path tnum rnum\n",
           g_progname);
-  fprintf(stderr, "  %s typical [-tl] [-td|-tb] [-nl|-nb] [-nc] path tnum rnum"
-          " [bnum [apow [fpow]]]\n", g_progname);
+  fprintf(stderr, "  %s typical [-tl] [-td|-tb] [-rc num] [-nl|-nb] [-nc] [-rr num]"
+          " path tnum rnum [bnum [apow [fpow]]]\n", g_progname);
   fprintf(stderr, "\n");
   exit(1);
 }
@@ -155,23 +157,23 @@ static void eprint(TCHDB *hdb, const char *func){
 
 /* print members of hash database */
 static void mprint(TCHDB *hdb){
-  if(hdb->cnt_writerec >= 0){
-    iprintf("bucket number: %lld\n", (long long)tchdbbnum(hdb));
-    iprintf("used bucket number: %lld\n", (long long)tchdbbnumused(hdb));
-    iprintf("cnt_writerec: %lld\n", (long long)hdb->cnt_writerec);
-    iprintf("cnt_reuserec: %lld\n", (long long)hdb->cnt_reuserec);
-    iprintf("cnt_moverec: %lld\n", (long long)hdb->cnt_moverec);
-    iprintf("cnt_readrec: %lld\n", (long long)hdb->cnt_readrec);
-    iprintf("cnt_searchfbp: %lld\n", (long long)hdb->cnt_searchfbp);
-    iprintf("cnt_insertfbp: %lld\n", (long long)hdb->cnt_insertfbp);
-    iprintf("cnt_splicefbp: %lld\n", (long long)hdb->cnt_splicefbp);
-    iprintf("cnt_dividefbp: %lld\n", (long long)hdb->cnt_dividefbp);
-    iprintf("cnt_mergefbp: %lld\n", (long long)hdb->cnt_mergefbp);
-    iprintf("cnt_reducefbp: %lld\n", (long long)hdb->cnt_reducefbp);
-    iprintf("cnt_appenddrp: %lld\n", (long long)hdb->cnt_appenddrp);
-    iprintf("cnt_deferdrp: %lld\n", (long long)hdb->cnt_deferdrp);
-    iprintf("cnt_flushdrp: %lld\n", (long long)hdb->cnt_flushdrp);
-  }
+  if(hdb->cnt_writerec < 0) return;
+  iprintf("bucket number: %lld\n", (long long)tchdbbnum(hdb));
+  iprintf("used bucket number: %lld\n", (long long)tchdbbnumused(hdb));
+  iprintf("cnt_writerec: %lld\n", (long long)hdb->cnt_writerec);
+  iprintf("cnt_reuserec: %lld\n", (long long)hdb->cnt_reuserec);
+  iprintf("cnt_moverec: %lld\n", (long long)hdb->cnt_moverec);
+  iprintf("cnt_readrec: %lld\n", (long long)hdb->cnt_readrec);
+  iprintf("cnt_searchfbp: %lld\n", (long long)hdb->cnt_searchfbp);
+  iprintf("cnt_insertfbp: %lld\n", (long long)hdb->cnt_insertfbp);
+  iprintf("cnt_splicefbp: %lld\n", (long long)hdb->cnt_splicefbp);
+  iprintf("cnt_dividefbp: %lld\n", (long long)hdb->cnt_dividefbp);
+  iprintf("cnt_mergefbp: %lld\n", (long long)hdb->cnt_mergefbp);
+  iprintf("cnt_reducefbp: %lld\n", (long long)hdb->cnt_reducefbp);
+  iprintf("cnt_appenddrp: %lld\n", (long long)hdb->cnt_appenddrp);
+  iprintf("cnt_deferdrp: %lld\n", (long long)hdb->cnt_deferdrp);
+  iprintf("cnt_flushdrp: %lld\n", (long long)hdb->cnt_flushdrp);
+  iprintf("cnt_adjrecc: %lld\n", (long long)hdb->cnt_adjrecc);
 }
 
 
@@ -181,6 +183,13 @@ static int myrand(int range){
 }
 
 
+/* get a random number based on normal distribution */
+static int myrandnd(int range){
+  int num = (int)tcdrandnd(range >> 1, range / 10);
+  return (num < 0 || num >= range) ? 0 : num;
+}
+
+
 /* parse arguments of write command */
 static int runwrite(int argc, char **argv){
   char *path = NULL;
@@ -244,12 +253,16 @@ static int runwrite(int argc, char **argv){
 static int runread(int argc, char **argv){
   char *path = NULL;
   char *tstr = NULL;
+  int rcnum = 0;
   int omode = 0;
   bool wb = false;
   bool rnd = false;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-nl")){
+      if(!strcmp(argv[i], "-rc")){
+        if(++i >= argc) usage();
+        rcnum = atoi(argv[i]);
+      } else if(!strcmp(argv[i], "-nl")){
         omode |= HDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
         omode |= HDBOLCKNB;
@@ -271,7 +284,7 @@ static int runread(int argc, char **argv){
   if(!path || !tstr) usage();
   int tnum = atoi(tstr);
   if(tnum < 1) usage();
-  int rv = procread(path, tnum, omode, wb, rnd);
+  int rv = procread(path, tnum, rcnum, omode, wb, rnd);
   return rv;
 }
 
@@ -362,7 +375,9 @@ static int runtypical(int argc, char **argv){
   char *astr = NULL;
   char *fstr = NULL;
   int opts = 0;
+  int rcnum = 0;
   int omode = 0;
+  int rratio = -1;
   bool nc = false;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
@@ -372,12 +387,18 @@ static int runtypical(int argc, char **argv){
         opts |= HDBTDEFLATE;
       } else if(!strcmp(argv[i], "-tb")){
         opts |= HDBTTCBS;
+      } else if(!strcmp(argv[i], "-rc")){
+        if(++i >= argc) usage();
+        rcnum = atoi(argv[i]);
       } else if(!strcmp(argv[i], "-nl")){
         omode |= HDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
         omode |= HDBOLCKNB;
       } else if(!strcmp(argv[i], "-nc")){
         nc = true;
+      } else if(!strcmp(argv[i], "-rr")){
+        if(++i >= argc) usage();
+        rratio = atoi(argv[i]);
       } else {
         usage();
       }
@@ -404,7 +425,7 @@ static int runtypical(int argc, char **argv){
   int bnum = bstr ? atoi(bstr) : -1;
   int apow = astr ? atoi(astr) : -1;
   int fpow = fstr ? atoi(fstr) : -1;
-  int rv = proctypical(path, tnum, rnum, bnum, apow, fpow, opts, omode, nc);
+  int rv = proctypical(path, tnum, rnum, bnum, apow, fpow, opts, rcnum, omode, nc, rratio);
   return rv;
 }
 
@@ -427,10 +448,6 @@ static int procwrite(const char *path, int tnum, int rnum, int bnum, int apow, i
     eprint(hdb, "tchdbtune");
     err = true;
   }
-  if(!tchdbsetcache(hdb, 1000)){
-    eprint(hdb, "tchdbsetcache");
-    err = true;
-  }
   if(!tchdbopen(hdb, path, HDBOWRITER | HDBOCREAT | HDBOTRUNC | omode)){
     eprint(hdb, "tchdbopen");
     err = true;
@@ -476,16 +493,16 @@ static int procwrite(const char *path, int tnum, int rnum, int bnum, int apow, i
     err = true;
   }
   tchdbdel(hdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
 
 
 /* perform read command */
-static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
-  iprintf("<Reading Test>\n  path=%s  tnum=%d  omode=%d  wb=%d  rnd=%d\n\n",
-          path, tnum, omode, wb, rnd);
+static int procread(const char *path, int tnum, int rcnum, int omode, bool wb, bool rnd){
+  iprintf("<Reading Test>\n  path=%s  tnum=%d  rcnum=%d  omode=%d  wb=%d  rnd=%d\n\n",
+          path, tnum, rcnum, omode, wb, rnd);
   bool err = false;
   double stime = tctime();
   TCHDB *hdb = tchdbnew();
@@ -494,7 +511,7 @@ static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
     eprint(hdb, "tchdbsetmutex");
     err = true;
   }
-  if(!tchdbsetcache(hdb, 1000)){
+  if(!tchdbsetcache(hdb, rcnum)){
     eprint(hdb, "tchdbsetcache");
     err = true;
   }
@@ -544,7 +561,7 @@ static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
     err = true;
   }
   tchdbdel(hdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -561,10 +578,6 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
     eprint(hdb, "tchdbsetmutex");
     err = true;
   }
-  if(!tchdbsetcache(hdb, 1000)){
-    eprint(hdb, "tchdbsetcache");
-    err = true;
-  }
   if(!tchdbopen(hdb, path, HDBOWRITER | omode)){
     eprint(hdb, "tchdbopen");
     err = true;
@@ -609,7 +622,7 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
     err = true;
   }
   tchdbdel(hdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -725,7 +738,7 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
     err = true;
   }
   tchdbdel(hdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -733,9 +746,10 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
 
 /* perform typical command */
 static int proctypical(const char *path, int tnum, int rnum, int bnum, int apow, int fpow,
-                       int opts, int omode, bool nc){
+                       int opts, int rcnum, int omode, bool nc, int rratio){
   iprintf("<Typical Access Test>\n  path=%s  tnum=%d  rnum=%d  bnum=%d  apow=%d  fpow=%d"
-          "  opts=%d  omode=%d  nc=%d\n\n", path, tnum, rnum, bnum, apow, fpow, opts, omode, nc);
+          "  opts=%d  rcnum=%d  omode=%d  nc=%d  rratio=%d\n\n",
+          path, tnum, rnum, bnum, apow, fpow, opts, rcnum, omode, nc, rratio);
   bool err = false;
   double stime = tctime();
   TCHDB *hdb = tchdbnew();
@@ -748,7 +762,7 @@ static int proctypical(const char *path, int tnum, int rnum, int bnum, int apow,
     eprint(hdb, "tchdbtune");
     err = true;
   }
-  if(!tchdbsetcache(hdb, rnum)){
+  if(!tchdbsetcache(hdb, rcnum)){
     eprint(hdb, "tchdbsetcache");
     err = true;
   }
@@ -762,6 +776,7 @@ static int proctypical(const char *path, int tnum, int rnum, int bnum, int apow,
     targs[0].hdb = hdb;
     targs[0].rnum = rnum;
     targs[0].nc = nc;
+    targs[0].rratio = rratio;
     targs[0].id = 0;
     if(threadtypical(targs) != NULL) err = true;
   } else {
@@ -769,6 +784,7 @@ static int proctypical(const char *path, int tnum, int rnum, int bnum, int apow,
       targs[i].hdb = hdb;
       targs[i].rnum = rnum;
       targs[i].nc = nc;
+      targs[i].rratio= rratio;
       targs[i].id = i;
       if(pthread_create(threads + i, NULL, threadtypical, targs + i) != 0){
         eprint(hdb, "pthread_create");
@@ -795,7 +811,7 @@ static int proctypical(const char *path, int tnum, int rnum, int bnum, int apow,
     err = true;
   }
   tchdbdel(hdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -847,7 +863,7 @@ static void *threadread(void *targ){
   int base = id * rnum;
   for(int i = 1; i <= rnum && !err; i++){
     char kbuf[RECBUFSIZ];
-    int ksiz = sprintf(kbuf, "%08d", base + (rnd ? myrand(i / 10 + 1) : i));
+    int ksiz = sprintf(kbuf, "%08d", base + (rnd ? myrandnd(i) : i));
     int vsiz;
     if(wb){
       char vbuf[RECBUFSIZ];
@@ -968,17 +984,31 @@ static void *threadwicked(void *targ){
       break;
     case 6:
       if(id == 0) putchar('6');
-      if(!tchdbputasync(hdb, kbuf, ksiz, vbuf, vsiz)){
-        eprint(hdb, "tchdbputasync");
-        err = true;
+      if(i > rnum / 4 * 3){
+        if(!tchdbputasync(hdb, kbuf, ksiz, vbuf, vsiz)){
+          eprint(hdb, "tchdbputasync");
+          err = true;
+        }
+      } else {
+        if(!tchdbput(hdb, kbuf, ksiz, vbuf, vsiz)){
+          eprint(hdb, "tchdbput");
+          err = true;
+        }
       }
       if(!nc) tcmapput(map, kbuf, ksiz, vbuf, vsiz);
       break;
     case 7:
       if(id == 0) putchar('7');
-      if(!tchdbputasync2(hdb, kbuf, vbuf)){
-        eprint(hdb, "tchdbputasync2");
-        err = true;
+      if(i > rnum / 4 * 3){
+        if(!tchdbputasync2(hdb, kbuf, vbuf)){
+          eprint(hdb, "tchdbputasync2");
+          err = true;
+        }
+      } else {
+        if(!tchdbput2(hdb, kbuf, vbuf)){
+          eprint(hdb, "tchdbput2");
+          err = true;
+        }
       }
       if(!nc) tcmapput2(map, kbuf, vbuf);
       break;
@@ -1087,7 +1117,7 @@ static void *threadwicked(void *targ){
       break;
     default:
       if(id == 0) putchar('@');
-      if(myrand(10000) == 0) srand((unsigned int)(tctime() * 100) % UINT_MAX);
+      if(myrand(10000) == 0) srand((unsigned int)(tctime() * 1000) % UINT_MAX);
       break;
     }
     if(!nc) tcglobalmutexunlock();
@@ -1114,14 +1144,16 @@ static void *threadtypical(void *targ){
   TCHDB *hdb = ((TARGTYPICAL *)targ)->hdb;
   int rnum = ((TARGTYPICAL *)targ)->rnum;
   bool nc = ((TARGTYPICAL *)targ)->nc;
+  int rratio = ((TARGTYPICAL *)targ)->rratio;
   int id = ((TARGTYPICAL *)targ)->id;
   bool err = false;
   TCMAP *map = (!nc && id == 0) ? tcmapnew2(rnum + 1) : NULL;
   int base = id * rnum;
+  int mrange = tclmax(50 + rratio, 100);
   for(int i = 1; !err && i <= rnum; i++){
     char buf[RECBUFSIZ];
-    int len = sprintf(buf, "%08d", base + myrand(i));
-    int rnd = myrand(100);
+    int len = sprintf(buf, "%08d", base + myrandnd(i));
+    int rnd = myrand(mrange);
     if(rnd < 10){
       if(!tchdbput(hdb, buf, len, buf, len)){
         eprint(hdb, "tchdbput");
@@ -1141,9 +1173,16 @@ static void *threadtypical(void *targ){
       }
       if(map) tcmapputcat(map, buf, len, buf, len);
     } else if(rnd < 25){
-      if(!tchdbputasync(hdb, buf, len, buf, len)){
-        eprint(hdb, "tchdbputasync");
-        err = true;
+      if(i > rnum / 10 * 9){
+        if(!tchdbputasync(hdb, buf, len, buf, len)){
+          eprint(hdb, "tchdbputasync");
+          err = true;
+        }
+      } else {
+        if(!tchdbput(hdb, buf, len, buf, len)){
+          eprint(hdb, "tchdbput");
+          err = true;
+        }
       }
       if(map) tcmapput(map, buf, len, buf, len);
     } else if(rnd < 30){
