@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The test cases of the hash database API
- *                                                      Copyright (C) 2006-2007 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
  * This file is part of Tokyo Cabinet.
  * Tokyo Cabinet is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -55,7 +55,7 @@ int main(int argc, char **argv){
   g_dbgfd = -1;
   const char *ebuf = getenv("TCDBGFD");
   if(ebuf) g_dbgfd = atoi(ebuf);
-  srand((unsigned int)(tctime() * 100) % UINT_MAX);
+  srand((unsigned int)(tctime() * 1000) % UINT_MAX);
   if(argc < 2) usage();
   int rv = 0;
   if(!strcmp(argv[1], "write")){
@@ -132,6 +132,7 @@ static void mprint(TCHDB *hdb){
   iprintf("cnt_appenddrp: %lld\n", (long long)hdb->cnt_appenddrp);
   iprintf("cnt_deferdrp: %lld\n", (long long)hdb->cnt_deferdrp);
   iprintf("cnt_flushdrp: %lld\n", (long long)hdb->cnt_flushdrp);
+  iprintf("cnt_adjrecc: %lld\n", (long long)hdb->cnt_adjrecc);
 }
 
 
@@ -466,7 +467,7 @@ static int procwrite(const char *path, int rnum, int bnum, int apow, int fpow,
     err = true;
   }
   tchdbdel(hdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -528,7 +529,7 @@ static int procread(const char *path, bool mt, int rcnum, int omode, bool wb){
     err = true;
   }
   tchdbdel(hdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -576,7 +577,7 @@ static int procremove(const char *path, bool mt, int rcnum, int omode){
     err = true;
   }
   tchdbdel(hdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -645,7 +646,7 @@ static int procrcat(const char *path, int rnum, int bnum, int apow, int fpow,
     err = true;
   }
   tchdbdel(hdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -1085,7 +1086,7 @@ static int procmisc(const char *path, int rnum, bool mt, int opts, int omode){
     err = true;
   }
   tchdbdel(hdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
@@ -1303,7 +1304,21 @@ static int procwicked(const char *path, int rnum, bool mt, int opts, int omode){
       break;
     default:
       putchar('@');
-      if(myrand(10000) == 0) srand((unsigned int)(tctime() * 100) % UINT_MAX);
+      if(myrand(10000) == 0) srand((unsigned int)(tctime() * 1000) % UINT_MAX);
+      if(myrand(rnum / 16 + 1) == 0){
+        int cnt = myrand(30);
+        for(int j = 0; j < rnum && !err; j++){
+          ksiz = sprintf(kbuf, "%d", i + j);
+          if(tchdbout(hdb, kbuf, ksiz)){
+            cnt--;
+          } else if(tchdbecode(hdb) != TCENOREC){
+            eprint(hdb, "tcbdbout");
+            err = true;
+          }
+          tcmapout(map, kbuf, ksiz);
+          if(cnt < 0) break;
+        }
+      }
       break;
     }
     if(i % 50 == 0) iprintf(" (%08d)\n", i);
@@ -1414,7 +1429,7 @@ static int procwicked(const char *path, int rnum, bool mt, int opts, int omode){
     err = true;
   }
   tchdbdel(hdb);
-  iprintf("time: %.3f\n", (tctime() - stime) / 1000);
+  iprintf("time: %.3f\n", tctime() - stime);
   iprintf("%s\n\n", err ? "error" : "ok");
   return err ? 1 : 0;
 }
